use std::{fs::File, io::Read};
use env_logger::Builder;
use log::{error, info, LevelFilter};
use rkyv::{deserialize, rancor::Error, Archive, Deserialize, Serialize};

#[derive(Archive, Deserialize, Serialize, Debug, PartialEq)]
#[rkyv(compare(PartialEq), derive(Debug))]
struct ChunkHeader {
    chunk_type: [u8; 4],
    end_of_chunk: u32,
    children_count: u32
}

#[derive(Archive, Deserialize, Serialize, Debug, PartialEq)]
#[rkyv(compare(PartialEq), derive(Debug))]
struct Database {
    unknown: u32,
    version: u32,
    children_count: u32,
    id: u64,
    unknown1: [u8; 32],
    language: u16,
    persistent_id: u64,
    hash: [u8; 20]
}

#[derive(Archive, Deserialize, Serialize, Debug, PartialEq)]
#[rkyv(compare(PartialEq), derive(Debug))]
struct DataSet {
    data_type: u32
}

#[derive(Archive, Deserialize, Serialize, Debug, PartialEq)]
#[rkyv(compare(PartialEq), derive(Debug))]
struct AlbumItem {
    chunk_type: [u8; 4],
    end_of_chunk: u32,
    total_length: u32,
    number_of_strings: u32,
    unknown: u16,
    album_id_for_track: u16,
    timestamp: u64,
    unknown1: u32
}

enum ChunkState {
    Header,
    Data
}

fn db(data: &[u8]) {
    let mut state = ChunkState::Header;
    let mut chunk_header = None;
    let mut i = 0;
    while i < data.len() {
        state = match state {
            ChunkState::Header => {
                chunk_header = Some(rkyv::access::<ArchivedChunkHeader, Error>(&data[i..i+12]).unwrap());
                info!("header: {:?}", chunk_header);
                i += 12;
                ChunkState::Data
            },
            ChunkState::Data => {
                let header = chunk_header.unwrap();
                match header.chunk_type {
                    [109, 104, 98, 100] => { //mhbd
                        info!("val: {:?}", rkyv::access::<ArchivedDatabase, Error>(&data[i..i+82]).unwrap());
                    },
                    [109, 104, 115, 100] => { //mhsd
                        info!("val: {:?}", rkyv::access::<ArchivedDataSet, Error>(&data[i..i+4]).unwrap());
                    },
                    [109, 104, 105, 97] => { // mhla

                    },
                    [109, 104, 108, 97] => { // mhia
                        info!("val: {:?}", rkyv::access::<ArchivedAlbumItem, Error>(&data[i..i+32]).unwrap());
                    }
                    _ => return
                }
                i += usize::try_from(header.end_of_chunk).unwrap() - 12;
                chunk_header = None;
                ChunkState::Header
            }
        }
    }
}

fn main() {

    // Initialize the logger with 'info' as the default level
    Builder::new()
        .filter(None, LevelFilter::Info)
        .init();

    let mut f = File::open("D:\\Documents\\iTunes\\iTunesDB").unwrap();
    let mut buf = Vec::new();
    match f.read_to_end(&mut buf) {
        Ok(n) => {
            let data = &buf[..n];
            db(data);
        },
        Err(e) => {
            error!("Error: {}",e);
        }
    }
}
